#!/usr/bin/perl  
use strict;
use myUtil;
use rtdb;
use bugdb;

use lib "/opt/rt3/lib";
use lib "/usr/lib/rt";
use RT;
RT::LoadConfig();
my $dbpass;
if (@ARGV < 1) {
	$dbpass = '';
}
else {
	$dbpass = $ARGV[0];
}

my $rtdb = rtdb->new(
		database => $RT::DatabaseName,
		user     => 'root',
		password => $dbpass
		);
my $bugdb = bugdb->new(
		database => 'bugs',
		user     => 'root',
		password => $dbpass
		);

# This functon is used to create 
sub create_map_tables {
	$rtdb->execSQL("DROP TABLE IF EXISTS group_bug2rt",0);
	$rtdb->execSQL("create table group_bug2rt(id int(11) NOT NULL auto_increment, ".
		"bug_group_id int(11) NOT NULL, rt_group_id int(11) NOT NULL, PRIMARY key (id))",0);
	$rtdb->execSQL("DROP TABLE IF EXISTS user_bug2rt",0);
	$rtdb->execSQL("create table user_bug2rt(id int(11) NOT NULL auto_increment, ".
		"bug_user_id int(11) NOT NULL, rt_user_id int(11) NOT NULL, PRIMARY key (id))",0);
}

sub insert_user_id {
	my $rows = $bugdb->fetch_hash_SQL(
		"select userid, login_name from profiles order by userid");
	for my $row (@$rows) {
		my $bug_id = $row->{'userid'};
		my $name = $row->{'login_name'};
		my $rt_id = $rtdb->get_user_id_by_email($name);
		my $sql = "insert into user_bug2rt(bug_user_id,rt_user_id) values('".
			"$bug_id', '$rt_id')";
		$rtdb->execSQL($sql,0);
	}
}

sub insert_group_id {
	my $rows = $bugdb->fetch_hash_SQL(
		"select id, name from groups order by id");
	for my $row (@$rows){
		my $bug_id = $row->{'id'};
		my $name = $row->{'name'};
		my $rt_id = get_rt_group_id_by_name($name);
		my $sql = "insert into group_bug2rt(bug_group_id,rt_group_id)"
			." values('$bug_id','$rt_id')";
		$rtdb->execSQL($sql,0);
	}
}


sub get_rt_group_id_by_name {
	my $name = shift;
	my $rows = $rtdb->fetch_hash_SQL(
		"select id from Groups where Name='$name'");
	return $rows->[0]->{id} if exists $rows->[0];
}

sub grant_right()
{
	my $rows = $rtdb->fetch_hash_SQL(
	      'SELECT id,EmailAddress FROM Users ORDER BY id');
	print 'Granting right...';
	for my $row (@$rows) {
		my $id = $row->{id};
		unless (myUtil::cc_type($row->{EmailAddress}) eq 'AdminCc') {
			# grant right to external address conditionally
			my $b_uid = $rtdb->get_bug_user_id_by_user_id($id);
			next unless $b_uid;
			my $subrows = $bugdb->fetch_hash_SQL(
			"SELECT bug_id FROM bugs WHERE assigned_to=$b_uid");
			next if (@$subrows eq 0);
		}
		print "$id.";
		my $grp = $rtdb->fetch_hash_SQL(
	      "SELECT id FROM Groups WHERE Instance=$id AND Domain='ACLEquivalence'");
		for my $r (@$grp) {
			my $gid = $r->{id};
			# grant global right
			$rtdb->execSQL(
	      "INSERT INTO ACL(PrincipalType, PrincipalId, RightName, ObjectType)"
	      ." VALUES('Group', '$gid', 'OwnTicket', 'RT::System')");
		}
	}

	print "done.\n";
}

sub add_member {
	my $rows = $rtdb->fetch_hash_SQL(
		"SELECT bug_user_id, rt_user_id FROM user_bug2rt "
		."ORDER by bug_user_id");
	for my $row (@$rows) {
		my $buid = $row->{bug_user_id};
		my $grps = $bugdb->fetch_hash_SQL(
			"SELECT group_id FROM user_group_map "
			."WHERE user_id=$buid and isbless=0 and isderived=0");
		for my $g_hash (@$grps) {
			my $rt_gid = $rtdb->fetch_hash_SQL(
				"SELECT rt_group_id FROM group_bug2rt "
				."WHERE bug_group_id=$g_hash->{group_id}");
			$rt_gid = $rt_gid->[0]->{rt_group_id};
			$rtdb->execSQL(
				"INSERT INTO GroupMembers(GroupId,MemberId) "
				."VALUES($rt_gid,$row->{rt_user_id})");
		}
	}
}

sub run {
	create_map_tables();
	insert_user_id();	
	insert_group_id();	
	grant_right();
	add_member();
}
run();

$bugdb->disconnect();
$rtdb->disconnect();
