#!/usr/bin/perl  
use strict;
use myUtil;
use rtdb;
use bugdb;

use lib "/opt/rt3/lib";
use lib "/usr/lib/rt";
use RT;
RT::LoadConfig();
my $dbpass;
if (@ARGV < 1) {
	$dbpass = '';
}
else {
	$dbpass = $ARGV[0];
}

my $rtdb = rtdb->new(
		database => $RT::DatabaseName,
		user     => 'root',
		password => $dbpass
		);
my $bugdb = bugdb->new(
		database => 'bugs',
		user     => 'root',
		password => $dbpass
		);

sub get_cc {
	my $bug = shift;
	my @cc = ();
	my @admincc = ();

	my $rows = $bugdb->fetch_hash_SQL(
			"SELECT who FROM cc WHERE bug_id=$bug");
	for my $row (@$rows) {
		my $who = $row->{who};
		$who = $rtdb->get_user_id_by_bug_user_id($who);
		my $email = $rtdb->get_email_by_user_id($who);
		if (myUtil::cc_type($email) eq 'AdminCc') {
			push @admincc, $email;
		} else {
			push @cc, $email;
		}
	}
	return (\@cc, \@admincc);
}

sub insert_ObjectCustomFields {
	my $id = $rtdb->get_root_id();
	my %cf_map = $rtdb->customField_map();
	for my $key (keys %cf_map) {
		my $rows = $rtdb->fetch_hash_SQL("select id from Queues");
		for my $q (@$rows) {
			my $so = $cf_map{$key}->[1];
			my $cf_id = $cf_map{$key}->[2];
			my $ins_sql = "INSERT into ObjectCustomFields"
				."(CustomField,ObjectId,SortOrder,Creator,"
				."Created,LastUpdatedBy,LastUpdated)"
				." VALUES"
				."('$cf_id','$q->{id}','$so','$id',"
				."now(),'$id',now())";
			$rtdb->execSQL($ins_sql,0);
		}
	}
}

sub insert_customFields {
	my %map = $rtdb->customField_map();

	for my $name (keys %map) {
		my $type;
		my $max = 1;
		my $lookup_type = 'RT::Queue-RT::Ticket';

		if ($name eq 'version' or
		    $name eq 'bug_severity' or
		    $name eq 'op_sys' or
		    $name eq 'rep_platform') {
			$type = 'Select';
		} else {
			$type = 'Freeform';
		}
		$rtdb->insert_customFields(
			Name      => $map{$name}->[0],
			Type      => $type,
			MaxValues => $max,
			LookupType=> $lookup_type
			);
	}
}

sub insert_CustomFieldValues {
	my %cf_map = $rtdb->customField_map();
	for my $field qw(bug_severity version rep_platform op_sys) {
		if (exists $cf_map{$field}) {
			my @values = $bugdb->fetch_array_SQL("SELECT $field ".
				"FROM bugs GROUP by $field ORDER by $field");
			$rtdb->insert_CustomFieldValues($cf_map{$field}->[0],
				@values);
		}
	}
}


#
sub print_bugs {
	my $rows = $bugdb->fetch_hash_SQL(
		"select * from bugs order by bug_id");
	my %cf_name2id = $rtdb->customField_map();
	create_map_com2ticket();
	my $root_id = $rtdb->get_root_id();
        print "\@Tickets = (\n";
        my $i = 0;
        for my $row (@$rows) {
		my $pri = myUtil::priority_bugzilla2rt($row->{priority});
		my $st = myUtil::status_bugzilla2rt($row->{bug_status});
                $i = $i +1;
                if ($i > 1) {
                        print "\t},\n";
                }
		my $q_id = $rtdb->get_queue_id_by_prod($row->{product_id});
		my $o_id = $rtdb->get_user_id_by_bug_user_id(
				$row->{'assigned_to'});
		my $req_id = $rtdb->get_user_id_by_bug_user_id($row->{reporter});
                print "\t{ id\t=> '$row->{'bug_id'}',\n";
                print "\t  Owner\t=> '$o_id',\n";
		print "\t  Requestor\t=> '$req_id',\n";
		my ($cc, $admincc) = get_cc($row->{bug_id});
		my $str = myUtil::list2qstring(@$cc);
		print "\t  Cc\t=> [$str],\n" if (@$cc > 0);
		$str = myUtil::list2qstring(@$admincc);
		print "\t  AdminCc\t=> [$str],\n" if (@$admincc > 0);
                print "\t  Status\t=> '$st',\n";
                print "\t  Subject\t=> ".$rtdb->quote($row->{'short_desc'}).",\n";
                print "\t  InitialPriority\t=> '$pri',\n";
                print "\t  FinalPriority\t=> '$pri',\n";
                print "\t  Priority\t=> '$pri',\n";
                print "\t  Queue\t=> '$q_id',\n";

		# adjust status according to resolution
		my $res = $row->{'resolution'};
		if ($res eq 'FIXED') {
			print "\t  Resolution\t=> '0',\n";
		} elsif ($res eq 'MOVED') {
			print "\t  Status\t=> 'deleted',\n";
		} elsif ($res eq 'INVALID') {
			print "\t  Status\t=> 'rejected',\n";
		} elsif ($res) {
			print "\t  Status\t=> 'stalled',\n";
		}
		my $minute = myUtil::hour2min($row->{'estimated_time'});
                print "\t  TimeEstimated\t=> '$minute',\n" if $minute > 0;
		$minute = myUtil::hour2min($row->{'remaining_time'});
                print "\t  TimeLeft\t=> '$minute',\n" if $minute > 0;
		my $iso_date = myUtil::iso_date($row->{deadline});
                print "\t  Due\t=> '$iso_date',\n" if $row->{deadline};
		$iso_date = myUtil::iso_date($row->{startdate});
                print "\t  Starts\t=> '$iso_date',\n" if $row->{startdate};
		$iso_date = $bugdb->get_started($row->{bug_id});
                print "\t  Started\t=> '$iso_date',\n" if defined $iso_date;
		$iso_date = $bugdb->get_resolved($row->{bug_id});
                print "\t  Resolved\t=> '$iso_date',\n";
                print "\t  Type\t=> 'ticket',\n";
                print "\t  Created\t=> '$row->{'creation_ts'}',\n";
                print "\t  Creator\t=> '$req_id',\n";
		for my $name (keys %cf_name2id) {
			my $cf_id = $cf_name2id{$name}->[2];
			my $v = $rtdb->quote($row->{$name}) if $row->{$name};
                	print "\t  'CustomField-$cf_id'\t=> $v,\n" if defined $v;
		}
#		(my $who, $iso_date) = $bugdb->get_lastupdate($row->{bug_id});
#		$who = $rtdb->get_user_id_by_bug_user_id($who) if $who;
#                print "\t  LastUpdated\t=> '$iso_date',\n";
#                print "\t  LastUpdatedBy\t=> '$who'";
        }
	$rows = $bugdb->fetch_hash_SQL(
		"select * from components order by product_id");
	for my $row (@$rows) {
                $i = $i +1;
                if ($i > 1) {
                        print ",\n\t},\n";
                }
                my $q_id = $rtdb->get_queue_id_by_prod($row->{'product_id'});
		my $created = 'now()';
		my $returned = $bugdb->get_comp_created($row->{product_id}, $row->{id});
		$created = $rtdb->quote($returned)
			if defined $returned;
                print "\t{ id\t=> '$i',\n";
                print "\t  Queue\t=> '$q_id',\n";
                print "\t  Subject\t=> ".$rtdb->quote("Component_".$row->{'name'}).",\n";
                print "\t  Type\t=> 'ticket',\n";
                print "\t  Created\t=> $created,\n";
                print "\t  Creator\t=> '$root_id',\n";
                #print "\t  LastUpdated\t=> 'now()',\n";
                #print "\t  LastUpdatedBy\t=> '$root_id'";
		insert_comp2ticket($i,$row->{'id'});
	}
        print ",}\n);\n";
}


sub create_map_com2ticket {
        $rtdb->execSQL("DROP TABLE IF EXISTS comp_bug2rt",0);
        $rtdb->execSQL("create table comp_bug2rt(id int(11) NOT NULL auto_increment, ".
                "comp_id int(11) NOT NULL, rt_ticket_id int(11) NOT NULL, PRIMARY key (id))",0);
}

sub insert_comp2ticket {
	my ($tk_id, $comp_id) = @_;
	my $sql = "insert into comp_bug2rt(comp_id,rt_ticket_id)"
		." values('$comp_id','$tk_id')";
	$rtdb->execSQL($sql,0);
}


sub run {
	insert_customFields();
	insert_CustomFieldValues();
	insert_ObjectCustomFields();
	print_bugs();
}
run();
#my $rows = $bugdb->fetch_hash_SQL("select * from profiles_activity");
#myUtil::print_rows($rows);
#my $string = "Don't view in monochrome (it looks 'fuzzy')!";
#my $quotedString = $bugdb->quote( $string );
#print "$quotedString\n";

$bugdb->disconnect();
$rtdb->disconnect();
