#!/usr/bin/perl  
use strict;
use myUtil;
use rtdb;
use bugdb;

use lib "/opt/rt3/lib";
use lib "/usr/lib/rt";
use RT;

$| = 1;    #unbuffer that output.

RT::LoadConfig();
my $dbpass;
if (@ARGV < 1) {
	$dbpass = '';
}
else {
	$dbpass = $ARGV[0];
}

my $rtdb = rtdb->new(
		database => $RT::DatabaseName,
		user     => 'root',
		password => $dbpass
		);
my $bugdb = bugdb->new(
		database => 'bugs',
		user     => 'root',
		password => $dbpass
		);


sub insert_customFields {
	$rtdb->insert_customFields(
		Name => 'Comment Type',
		Type => 'Select',
		LookupType => 'RT::Queue-RT::Ticket-RT::Transaction'
		);
}

sub insert_CustomFieldValues {
	my $cf_id = $rtdb->get_CustomField_id_by_name('Comment Type');
	my $id = $rtdb->get_root_id();
	my $sql = "select tag from comment_tags group by tag order by tag";
	my $i = 0;
	my $rows = $bugdb->fetch_hash_SQL($sql);
	for my $v (@$rows){
		my $sql2= "insert into CustomFieldValues(CustomField,Name,SortOrder,".
			"Creator,Created,LastUpdatedBy,LastUpdated) values(".
			"'$cf_id',".$rtdb->quote($v->{'tag'})
			.",'$i','$id',now(),'$id',now())";
		$rtdb->execSQL($sql2,0);
		$i = $i +1;
	}

}

sub Insert_ObjectCustomFields {
        my $id = $rtdb->get_root_id();
        my $sql = "select id,Name from CustomFields where LookupType='RT::Queue-RT::Ticket-RT::Transaction'".
                        " and Name='Comment Type' ";
        my $rows = $rtdb->fetch_hash_SQL($sql);
        for my $cf (@$rows){
                my $rows1 = $rtdb->fetch_hash_SQL("select id,Name from Queues");
                for my $q (@$rows1){
                        my $cf_id = $cf->{'id'};
                        my $so_id = 0;
                        my $ins_sql = "insert into ObjectCustomFields(CustomField," .
                                "ObjectId,SortOrder,Creator,Created,LastUpdatedBy,".
                                "LastUpdated) values('$cf_id','$q->{'id'}','$so_id','$id',now(),'$id',now())";
                        $rtdb->execSQL($ins_sql,0);
                }
        }
}


sub add_transaction {
	my %args = (
		ObjectType => 'RT::Ticket',
		@_);
	my $sql = "INSERT INTO Transactions(ObjectType,ObjectId,Type";
	$sql .= ",Field" if exists $args{Field};
	$sql .= ",OldValue" if exists $args{OldValue};
	$sql .= ",NewValue" if exists $args{NewValue};
	$sql .= ",TimeTaken" if exists $args{TimeTaken};
	$sql .= ",Creator,Created) VALUES(";
	$sql .= "'" . $args{ObjectType} . "',";
	$sql .= "'" . $args{ObjectId} . "',";
	$sql .= "'" . $args{Type} . "',";
	$sql .= "'" . $args{Field} . "'," if exists $args{Field};
	$sql .= $rtdb->quote($args{OldValue}) . "," if exists $args{OldValue};
	$sql .= $rtdb->quote($args{NewValue}) . "," if exists $args{NewValue};
	$sql .= "'" . $args{TimeTaken} . "'," if exists $args{TimeTaken};
	$sql .= "'" . $args{Creator} . "',";
	$sql .= "'" . $args{Created} . "')";
	$rtdb->execSQL($sql, 0);
}

sub set_init_prio {
	my ($ticket, $prio) = @_;
	$rtdb->execSQL(
	       "UPDATE Tickets set InitialPriority=$prio WHERE id=$ticket");
}

sub insert_bug_trans {
	my $url = "fsck.com-rt://$RT::rtname/ticket/";
	my %cf_map = $rtdb->customField_map();
	my $sql = "SELECT ba.bug_id,ba.attach_id,ba.who,ba.bug_when,ba.added,ba.removed,fd.name "
		."FROM bugs_activity ba,fielddefs fd "
		."WHERE ba.fieldid=fd.fieldid and fd.name in"
		."('deadline','startdate','estimated_time','remaining_time',"
		."'reporter','content','bug_status','assigned_to','dependson',"
		."'blocked','cc','priority'";

	$sql .= ',' . myUtil::list2qstring(keys %cf_map) unless (%cf_map == 0);
	$sql .= ") order by ba.bug_id";
	my $rows = $bugdb->fetch_hash_SQL($sql);
	my $lstbug = 0;
	my %init_set = ();
	for my $trans (@$rows) {
		my $type ='';
		my $creator = $rtdb->get_user_id_by_bug_user_id($trans->{who});
		if ( ($trans->{'name'} eq 'estimated_time') or
		    ($trans->{'name'} eq 'remaining_time')) {
			$type = 'Set';
			$trans->{added} = myUtil::hour2min($trans->{added});
			$trans->{removed} = myUtil::hour2min($trans->{removed});
			$trans->{name} = 'TimeEstimated'
				if $trans->{name} eq 'estimated_time';
			$trans->{name} = 'TimeLeft'
				if $trans->{name} eq 'remaining_time';
		} elsif (($trans->{'name'} eq 'deadline') or 
		    ($trans->{'name'} eq 'startdate')) {
			$type = 'Set';
			$trans->{name} = 'Due'
				if $trans->{name} eq 'deadline';
			$trans->{name} = 'Starts'
				if $trans->{name} eq 'startdate';

			$trans->{added} = myUtil::iso_date($trans->{added})
				if (length($trans->{added}) > 0);
			$trans->{removed} = myUtil::iso_date($trans->{removed})
				if (length($trans->{removed}) > 0);
		} elsif (($trans->{'name'} eq 'reporter') or
		    ($trans->{'name'} eq 'content')) {
			$type = 'Set';
			if ($trans->{name} eq 'reporter') {
				$trans->{name} = 'Requestor';
			}
		} elsif ($trans->{name} eq 'priority') {
			$type = 'Set';
			$trans->{name} = 'Priority';
			$trans->{removed} = myUtil::priority_bugzilla2rt($trans->{removed});
			$trans->{added} = myUtil::priority_bugzilla2rt($trans->{added});
			unless (exists $init_set{$trans->{bug_id}}) {
			    set_init_prio($trans->{bug_id}, $trans->{removed});
			    $init_set{$trans->{bug_id}} = 1;
			}
		} elsif ($trans->{'name'} eq 'bug_status') {
			$type = 'Status';
			$trans->{name} = 'Status';
			$trans->{added} = myUtil::status_bugzilla2rt($trans->{added});
			$trans->{removed} = myUtil::status_bugzilla2rt($trans->{removed});
		} elsif ($trans->{name} eq 'resolution') {
			$type = 'Status';
		} elsif ($trans->{'name'} eq 'assigned_to') {
			$trans->{name} = 'Owner';
			my $valid_add = 1;
			if (length($trans->{'added'}) > 0) {
				# added and removed are email address in bugzilla
				# but are user id in RT
				my $uid = $rtdb->get_user_id_by_email($trans->{added});
				unless (defined $uid) {
					warn "$trans->{added} not in profiles, ".
					     "but in transactions of ".
					     "$trans->{bug_id}, ignore it. Bugzilla bug?";
					$valid_add = 0;
				}
				$trans->{added} = $uid;
				if (length($trans->{removed})>0 and $valid_add){
					# both add and remove
					# for remove, the user must be existed
					$trans->{removed} = $rtdb->get_user_id_by_email($trans->{removed});
					if ($trans->{added} eq $creator) {
						$type = 'Steal';
					} elsif($trans->{removed} eq $creator) {
						$type = 'Give';
					} else {
						$type = 'Force';
					}
				} elsif ($valid_add) {
					# only add, originally no owner
					$type = 'Take';
				} elsif (length($trans->{removed}) > 0) {
					# only remove
					$type = 'Untake';
					$trans->{added} = '';
				} else {
					# nothing
					next;
				}
			} else {
				$type = 'Untake';
			}
		} elsif (($trans->{'name'} eq 'dependson') ||
			($trans->{'name'} eq 'blocked')) {
			# dependson and blocked are ,-seperated list of bugs
			if ($trans->{name} eq 'dependson') {
				$trans->{name} = 'HasMember';
			} else {
				$trans->{name} = 'MemberOf';
			}
			if (length($trans->{'added'}) > 0) {
				$type = 'AddLink';
				for my $bug (split /,\s*/, $trans->{added}) {
					add_transaction(
						ObjectId => $trans->{bug_id},
						Type     => $type,
						Field    => $trans->{name},
						NewValue => $url . $bug,
						Creator  => $creator,
						Created  => $trans->{bug_when}
					);
				}
			}
			if (length($trans->{'removed'}) > 0) {
				$type = 'DeleteLink';
				for my $bug (split /,\s*/, $trans->{removed}) {
					add_transaction(
						ObjectId => $trans->{bug_id},
						Type     => $type,
						Field    => $trans->{name},
						OldValue => $url . $bug,
						Creator  => $creator,
						Created  => $trans->{bug_when}
					);
				}
			}
			next;
		} elsif ($trans->{'name'} eq 'cc') {
			# cc is ,-seperated email addresses in bugzilla
			# user id in RT

			if (length($trans->{'added'}) > 0) {
				$type = 'AddWatcher';
				for my $email (split /,\s*/, $trans->{added}) {
					my $id = $rtdb->get_user_id_by_email($email);
					unless (defined $id) {
						warn "$email not in profiles, ".
						     "but in transactions of ".
						     "$trans->{bug_id}, ignore it. Bugzilla bug?";
						next;
					}
					my $field = myUtil::cc_type($email);
					add_transaction(
						ObjectId => $trans->{bug_id},
						Type     => $type,
						Field    => $field,
						NewValue => $id,
						Creator  => $creator,
						Created  => $trans->{bug_when}
						       );
				}
			}
			if (length($trans->{'removed'}) > 0) {
				$type = 'DelWatcher';
				for my $email (split /,\s*/, $trans->{removed}){
					# for remove, the user must be existed
					my $id = $rtdb->get_user_id_by_email($email);
					my $field = myUtil::cc_type($email);
					add_transaction(
						ObjectId => $trans->{bug_id},
						Type     => $type,
						Field    => $field,
						OldValue => $id,
						Creator  => $creator,
						Created  => $trans->{bug_when}
						       );
				}
			}
			next;
		} elsif ($trans->{'name'} eq 'short_desc') {
			$type = 'Subject';
			$trans->{name} = 'Subject';
		} elsif (exists $cf_map{$trans->{name}}) {
			$type = 'CustomField';
			$trans->{name} = $cf_map{$trans->{name}}->[0];
		} else {
			next;
		}

		add_transaction(
			ObjectType => 'RT::Ticket',
			ObjectId   => $trans->{bug_id},
			Type       => $type,
			Field      => $trans->{name},
			OldValue   => $trans->{removed},
			NewValue   => $trans->{added},
			Creator    => $creator,
			Created    => $trans->{bug_when}
			       );
		if ($lstbug ne $trans->{bug_id}) {
			if ($trans->{bug_id} % 500 eq 1) {
				print $trans->{bug_id};
			} else {
				print '.';
			}
		}
		$lstbug = $trans->{bug_id};
	}#end for
}


sub insert_attach {
	my $sel_sql = "select * from attachments order by bug_id";
	my $rows = $bugdb->fetch_hash_SQL($sel_sql);
	my $lstbug = 0;
        for my $att (@$rows){
		my $type = 'Correspond';
		$type = 'Comment' if ($att->{isprivate});
		my $creator = $rtdb->get_user_id_by_bug_user_id
				($att->{'submitter_id'});
		add_transaction(
			ObjectType => 'RT::Ticket',
			ObjectId   => $att->{bug_id},
			Type       => $type,
			Creator    => $creator,
			Created    => $att->{creation_ts}
			       );

		# If scrips are enabled while creating ticket,
		# attachments will be automatically created during creation time
		# cause attachments can't be added here

		my $tran_id = get_trans_max_id();
		my $att_sql = "INSERT into Attachments"
			     ."(id,TransactionId,Filename,Creator,"
			     ."Content,ContentType,Subject,Created) "
			     ."VALUES"
			     ."('$att->{'attach_id'}','$tran_id',"
			     .$rtdb->quote($att->{'filename'}) .",'$creator',"
			     .$rtdb->quote($att->{'thedata'})
			     .",".$rtdb->quote($att->{'mimetype'})
			     .",".$rtdb->quote($att->{'description'})
			     .",".$rtdb->quote($att->{'creation_ts'}).")";
		if ($lstbug ne $att->{bug_id}) {
			if ($att->{bug_id} % 500 eq 1) {
				print $att->{bug_id};
			} else {
				print '.';
			}
		}
		$lstbug = $att->{bug_id};
		$rtdb->execSQL($att_sql,0);
	}
}

sub insert_Links {
        my $url = "fsck.com-rt://$RT::rtname/ticket/";
        my $rows = $bugdb->fetch_hash_SQL("select * from dependencies");
        for my $row (@$rows) {
                my($time_date,$user_id) = $rtdb->get_lastupdate($row->{'bug_id'});
                my $localbase = $row->{'dependson'};
                my $localtarget = $row->{'blocked'};
                my $base = $url.$localbase;
                my $target = $url.$localtarget;
                my $sql = "INSERT into Links"
			."(Base,Target,Type,LocalTarget,"
			."LocalBase,LastUpdatedBy,LastUpdated,Creator,Created) "
			."VALUES"
			."('$base','$target','MemberOf','$localtarget',"
			."'$localbase','$user_id','$time_date','$user_id','$time_date')";
                $rtdb->execSQL($sql,0);
        }

}


sub insert_commet {
	my $sel_sql = "select * from longdescs order by bug_id";
        my $rows = $bugdb->fetch_hash_SQL($sel_sql);
	my $lstbug = 0;
	my $cf = $rtdb->get_CustomField_id_by_name('Comment Type');
        for my $att (@$rows){
                my $creator = $rtdb->get_user_id_by_bug_user_id($att->{who});
		my $type = 'Correspond';
		$type = 'Comment' if ($att->{isprivate});
		my $time = myUtil::hour2min($att->{work_time});
		add_transaction(
			ObjectType => 'RT::Ticket',
			ObjectId   => $att->{bug_id},
			Type       => $type,
			TimeTaken  => $time,
			Creator    => $creator,
			Created    => $att->{bug_when}
			       );
                my $tran_id = get_trans_max_id();
		my $tag = $bugdb->get_tag_by_comment_id($att->{comment_id});
		$rtdb->insert_ObjectCustomFieldValues(
				CustomField => $cf,
				ObjectType  => 'RT::Transaction',
				ObjectId    => $tran_id,
				Content     => $tag,
				Creator     => $creator,
				Created     => $att->{bug_when},
				LastUpdatedBy=> $creator,
				LastUpdated => $att->{bug_when}
				) if $tag;
		my $att_sql = "insert into Attachments(TransactionId,Creator,".
                                "Content,ContentType,Created) values(".
                                "'$tran_id',"."'$creator',"
				.$rtdb->quote($att->{thetext}).",'text/plain',"
				.$rtdb->quote($att->{'bug_when'}).")";
		if ($lstbug ne $att->{bug_id}) {
			if ($att->{bug_id} % 500 eq 1) {
				print $att->{bug_id};
			} else {
				print '.';
			}
		}
		$lstbug = $att->{bug_id};
                $rtdb->execSQL($att_sql,0);
	}
}


sub get_trans_max_id {
	my $rows = $rtdb->fetch_hash_SQL(
			"select id from Transactions order by id desc limit 1");
	return $rows->[0]->{id} if exists $rows->[0];
}


sub run {
	insert_customFields();
	insert_CustomFieldValues();
	Insert_ObjectCustomFields();
	print "start bug_trans\n";
	insert_bug_trans();
	print "end bug_trans\n";
	print "start attach\n";
	insert_attach();
	print "end attach\n";
	print "start comment\n";
	insert_commet();
	print "end comment\n";
	insert_Links();

}
run();


$bugdb->disconnect();
$rtdb->disconnect();
