#!/usr/bin/perl  
use strict;
use myUtil;
use rtdb;
use bugdb;

use lib "/opt/rt3/lib";
use lib "/usr/lib/rt";
use RT;

$| = 1;    #unbuffer that output.

RT::LoadConfig();
RT::Init();
my $dbpass;
if (@ARGV < 1) {
	$dbpass = '';
}
else {
	$dbpass = $ARGV[0];
}

my $rtdb = rtdb->new(
		database => $RT::DatabaseName,
		user     => 'root',
		password => $dbpass
		);
my $bugdb = bugdb->new(
		database => 'bugs',
		user     => 'root',
		password => $dbpass
		);

sub find_ticket_group
{
	my $ticket = shift;
	my $role = shift;

	my $rows = $rtdb->fetch_hash_SQL(
		"SELECT id FROM Groups WHERE Instance=$ticket and Type='$role'"
		." and Domain='RT::Ticket-Role'");
	return $rows->[0]->{id} if exists $rows->[0];

	warn "No $role group for ticket $ticket";
	return undef;
}


sub get_disable_by_user_id {
	my $user_id = shift;
	my $rows = $bugdb->fetch_hash_SQL(
		"select length(disabledtext) l from profiles where userid='$user_id'");
	return $rows->[0]->{'l'} if exists $rows->[0];
}


sub user_lst_updated {
	my $user = shift;

	my $rows = $bugdb->fetch_hash_SQL(
		"SELECT who,profiles_when FROM profiles_activity "
		."WHERE userid=$user ORDER by profiles_when desc limit 1");
	unless (exists $rows->[0]) {
		$rows = $bugdb->fetch_hash_SQL(
			"SELECT refreshed_when FROM profiles "
			."WHERE userid=$user");
		return ('', $rows->[0]->{refreshed_when});
	} else {
		return ($rows->[0]->{who}, $rows->[0]->{profiles_when});
	}
}

sub user_first_updated {
	my $user = shift;

	my $rows = $bugdb->fetch_hash_SQL(
		"SELECT who,profiles_when FROM profiles_activity "
		."WHERE userid=$user ORDER by profiles_when limit 1");
	unless (exists $rows->[0]) {
		$rows = $bugdb->fetch_hash_SQL(
			"SELECT refreshed_when FROM profiles "
			."WHERE userid=$user");
		return ('', $rows->[0]->{refreshed_when});
	} else {
		return ($rows->[0]->{who}, $rows->[0]->{profiles_when});
	}
}


sub update_user {
	print "Updating Users...";
        my $rows = $rtdb->fetch_hash_SQL(
		"select rt_user_id,bug_user_id from user_bug2rt");
        for my $row (@$rows){
		my $buid = $row->{bug_user_id};
                my $rt_user_id = $row->{'rt_user_id'};
                my $password = $bugdb->get_password_by_user_id($buid);
                my $disable_len = get_disable_by_user_id($buid);
		my ($updatedBy, $updated) = user_lst_updated($buid);
		unless ($updatedBy) {
			$updatedBy = $rtdb->get_root_id();
		} else {
			$updatedBy = $rtdb->get_user_id_by_bug_user_id($buid);
		}

		# Bugzilla doesn't tell us when and who created the users,
		# We just set them to firstUpdated and firstUpdatedBy
		my ($firstBy, $first) = user_first_updated($buid);
		unless ($firstBy) {
			$firstBy = $rtdb->get_root_id();
		} else {
			$firstBy = $rtdb->get_user_id_by_bug_user_id($buid);
		}

                my $sql = "UPDATE Users set Password=".$rtdb->quote($password)
			.", LastUpdatedBy='$updatedBy',LastUpdated='$updated' "
			.", Creator='$firstBy',Created='$first' "
			."WHERE id='$rt_user_id'";
                $rtdb->execSQL($sql,0);

		# Update Transactions
                $rtdb->execSQL(
			"UPDATE Transactions set Created='$first' ".
			"WHERE Type='Create' and ObjectType='RT::User' ".
			"AND ObjectId=$rt_user_id");
                if ($disable_len > 0) {
                        my $sql1 = "update Principals set Disabled='1' "
			     ."WHERE "
			     ."ObjectId='$rt_user_id' and PrincipalType='User'";
                        $rtdb->execSQL($sql1,0);
                }
#		print "$rt_user_id.";
        }
	print "done\n";
}

sub update_ticket_group {
	my $ticket = shift;
	my $created= shift;

	for my $role qw(Requestor Owner Cc AdminCc) {
		my $grp = find_ticket_group($ticket, $role);
		next unless defined $grp;

		# We don't reset the Creator of this transaction
		$rtdb->execSQL(
			"UPDATE Transactions set Created='$created' ".
			"WHERE ObjectType='RT::Group' and ObjectId=$grp ".
			"AND Type='Create'");
	}
}

sub update_component {
	my $rows = $rtdb->fetch_hash_SQL("SELECT rt_ticket_id from comp_bug2rt");
	for my $row (@$rows) {
		my $tid = $row->{rt_ticket_id};
		my @array = $rtdb->fetch_array_SQL(
			"SELECT Created from Tickets where id=$tid");
		$rtdb->execSQL("UPDATE Transactions set Created='$array[0]' ".
			"WHERE ObjectType='RT::Ticket' and ObjectId=$tid ".
			"AND Type='Create'");
	}
}

sub update_ticket {
	print "Updating Tickets...";
        my $sel_sql="select bug_id,reporter, creation_ts from bugs";
        my $rows = $bugdb->fetch_hash_SQL($sel_sql);
        for my $row (@$rows) {
                my $bug_id = $row->{'bug_id'};
		my $create_date = $row->{'creation_ts'};
		my $creator = $rtdb->get_user_id_by_bug_user_id($row->{reporter});

		# Transactions for ticket creation
                my $sql = "update Transactions set Creator='$creator".
			"',Created='$create_date'".
			" where ObjectType='RT::Ticket' and Type='Create'".
			" and ObjectId='$bug_id'";
                $rtdb->execSQL($sql,0);

		# Transactions for the creations of ticket-related groups
		update_ticket_group($bug_id, $create_date);

                my ($lstUpBy, $lstUpdated) = $rtdb->get_lastupdate($bug_id);

		my $timeworked = get_timeworked($bug_id);
                $sql = "update Tickets ".
		      "set LastUpdated='$lstUpdated',LastUpdatedBy='$lstUpBy'";

		$sql .= ",TimeWorked=$timeworked" if $timeworked > 0;
		$sql .= " where id='$bug_id'";
                $rtdb->execSQL($sql,0);
		if ($bug_id % 500 eq 1) {
                	print $bug_id;
		} else {
                	print '.';
		}
        }
	update_component();
	print "done\n";
}

# The last time the ticket's status was flipped from new to another status
sub get_started {
	my $bug_id = shift;
	my $sql =  "select Created from Transactions where Type='Status'".
			" and OldValue='new'".
			" and ObjectId = '$bug_id'".
			" order by Created desc limit 1";
	my $rows = $rtdb->fetch_hash_SQL($sql);
	return $rows->[0]->{Created} if exists $rows->[0];
	return undef;
}

# The last time that this ticket's status was set to
# resolved, rejected, deleted
sub get_resolved {
	my $bug_id = shift;
	my $sql =  "select Created from Transactions where Type='Status'".
			" and NewValue in('resolved','rejected','deleted')".
			" and ObjectId = '$bug_id'".
			" order by Created desc limit 1";
	my $rows = $rtdb->fetch_hash_SQL($sql);
	return $rows->[0]->{Created} if exists $rows->[0];
}

sub set_duplicate {
	print "Setting duplicates...";
	my $t = RT::Ticket->new($RT::SystemUser);
	my $rows = $bugdb->fetch_hash_SQL(
			"SELECT * FROM duplicates ORDER BY dupe");
	for my $row (@$rows) {
#		my $dup_of = $row->{dupe_of};
		my $dup    = $row->{dupe};
		$t->Load($dup);
		$t->MergeInto($row->{dupe_of});
#		print "$dup.";
#		$rtdb->execSQL(
#			"UPDATE Tickets set EffectiveId=$dup_of WHERE id=$dup");
		my $fetch = $bugdb->fetch_hash_SQL(
			"SELECT ba.bug_when,ba.who ".
			"FROM bugs_activity ba,fielddefs fd ".
			"WHERE ba.fieldid=fd.fieldid and fd.name='resolution' ".
			"AND ba.added='DUPLICATE' and ba.bug_id=$dup ".
			"ORDER by ba.bug_when desc limit 1");
		unless (exists $fetch->[0]) {
			warn "missing activity (DUPLICATE) for $dup";
			next;
		}
		my $when = $fetch->[0]->{bug_when};
		my $who = $fetch->[0]->{who};
		$who = $rtdb->get_user_id_by_bug_user_id($who);
		$rtdb->execSQL("UPDATE Transactions ".
			       "set Creator=$who,Created='$when' ".
			       "WHERE Type='AddLink' and Field='MergedInto' ".
			       "AND ObjectType='RT::Ticket' and ObjectId=$dup");
	}
	print "done.\n";
}

sub get_timeworked {
	my $bug = shift;

	my @rows = $rtdb->fetch_array_SQL(
		 "SELECT SUM(TimeTaken) FROM Transactions"
		." WHERE ObjectType='RT::Ticket' AND ObjectId=$bug");

	return $rows[0];
}


sub run {
	print "Post updating...\n";
	# duplicates should be set before updating
	set_duplicate();
	update_user();
	update_ticket();
	update_component();
}
run();


$bugdb->disconnect();
$rtdb->disconnect();
